﻿
using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using MindFusion.Gauges;

namespace GaugesStopwatch
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();

			var minutesScale = (OvalScale)stopwatch.GetElementByName("MinutesScale");
			minutesScale.QueryLabelValue += (s, e) =>
				{
					if (e.CalculatedLabelValue == 0)
						e.NewValue = 30;
				};
			var secondsScale = (OvalScale)stopwatch.GetElementByName("SecondsScale");
			secondsScale.QueryLabelValue += (s, e) =>
			{
				if (e.CalculatedLabelValue == 0)
					e.NewValue = 60;
			};
		}

		private void UpdateTime()
		{
			var now = DateTime.Now;
			elapsedTime += now - startTime;
			startTime = now;
		}

		private void UpdatePointers()
		{
			var minutesPointer = (Pointer)stopwatch.GetElementByName("MinutesPointer");
			var secondsPointer = (Pointer)stopwatch.GetElementByName("SecondsPointer");
			minutesPointer.Value = (float)elapsedTime.TotalMinutes;
			secondsPointer.Value = (float)Math.Round((float)(elapsedTime.TotalSeconds % 60), 1);
		}

		private void stopwatch_PrepaintPointer(object sender, PrepaintEventArgs e)
		{
			e.CancelDefaultPainting = true;

			var shadowFill = new SolidBrush(Color.FromArgb(30, Color.Black));
			Brush fill = new SolidBrush(Color.FromArgb(175, 8, 9));
			Brush fill2 = new SolidBrush(Color.FromArgb(170, 170, 170));
			Brush fill3 = new SolidBrush(Color.FromArgb(65, 65, 65));
			GraphicsState state = e.Graphics.Save();
			e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
			e.Graphics.ScaleTransform(e.Element.RenderSize.Width, e.Element.RenderSize.Height);

			if (e.Element.Name == "SecondsPointer")
			{
				var polygon = new PointF[]
					{
						new PointF(0.1f, 0.35f),
						new PointF(1, 0.45f),
						new PointF(1, 0.55f),
						new PointF(0.1f, 0.65f)
					};
				var ellipse1 = new RectangleF(0, 0, 0.2f, 1f);
				var ellipse2 = new RectangleF(0.04f, 0.2f, 0.12f, 0.6f);
				var ellipse3 = new RectangleF(0.08f, 0.4f, 0.04f, 0.2f);

				// Draw the pointer's shadow
				var state2 = e.Graphics.Save();
				e.Graphics.TranslateTransform(4, 4, MatrixOrder.Append);
				e.Graphics.FillEllipse(shadowFill, ellipse1);
				GraphicsPath p = new GraphicsPath();
				p.AddEllipse(ellipse1);
				e.Graphics.SetClip(p, CombineMode.Exclude);
				e.Graphics.FillPolygon(shadowFill, polygon);
				p.Dispose();
				e.Graphics.Restore(state2);

				// Draw the pointer itself
				e.Graphics.FillPolygon(fill, polygon);
				e.Graphics.FillEllipse(fill, ellipse1);
				e.Graphics.FillEllipse(fill2, ellipse2);
				e.Graphics.FillEllipse(fill3, ellipse3);
			}
			else if (e.Element.Name == "MinutesPointer")
			{
				var polygon = new PointF[]
					{
						new PointF(0.15f, 0.35f),
						new PointF(1, 0.45f),
						new PointF(1, 0.55f),
						new PointF(0.15f, 0.65f)
					};
				var ellipse1 = new RectangleF(0, 0, 0.3f, 1f);
				var ellipse2 = new RectangleF(0.06f, 0.2f, 0.18f, 0.6f);
				var ellipse3 = new RectangleF(0.12f, 0.4f, 0.06f, 0.2f);

				// Draw the pointer
				e.Graphics.FillPolygon(fill, polygon);
				e.Graphics.FillEllipse(fill, ellipse1);
				e.Graphics.FillEllipse(fill2, ellipse2);
				e.Graphics.FillEllipse(fill3, ellipse3);
			}

			e.Graphics.Restore(state);
			fill3.Dispose();
			fill2.Dispose();
			fill.Dispose();
		}

		private void stopwatch_PrepaintBackground(object sender, PrepaintEventArgs e)
		{
			e.CancelDefaultPainting = true;

			var bounds = new RectangleF(0, 0, e.Element.RenderSize.Width, e.Element.RenderSize.Height);

			var path = new GraphicsPath();
			path.AddEllipse(bounds);

			// White background
			e.Graphics.FillEllipse(Brushes.White, bounds);

			// Gradient background
			var brush = new PathGradientBrush(path);
			var colors = new ColorBlend();
			colors.Positions = new float[] { 0, 0.5f, 1 };
			colors.Colors = new Color[] { Color.FromArgb(30, Color.Black),
				Color.FromArgb(0, Color.Black), Color.FromArgb(0, Color.Black) };
			brush.InterpolationColors = colors;

			e.Graphics.FillPath(brush, path);

			brush.Dispose();

			// Chrome frame
			var innerBounds = bounds;
			innerBounds.Inflate(-10, -10);

			var innerPath = new GraphicsPath();
			innerPath.AddEllipse(innerBounds);

			var state = e.Graphics.Save();
			e.Graphics.SetClip(path, CombineMode.Intersect);
			e.Graphics.SetClip(innerPath, CombineMode.Exclude);

			var chromeBrush = new LinearGradientBrush(bounds, Color.Empty, Color.Empty, 135);
			colors = new ColorBlend();
			colors.Positions = new float[] { 0f, 0.3f, 0.5f, 0.7f, 1f };
			colors.Colors = new Color[] { Color.FromArgb(179, 179, 179), Color.FromArgb(179, 179, 179), Color.White,
				Color.FromArgb(179, 179, 179), Color.FromArgb(179, 179, 179) };
			chromeBrush.InterpolationColors = colors;

			e.Graphics.FillRectangle(chromeBrush, bounds);

			chromeBrush.Dispose();

			e.Graphics.Restore(state);

			e.Graphics.DrawEllipse(Pens.Black, bounds);
			e.Graphics.DrawEllipse(Pens.Black, innerBounds);

			innerPath.Dispose();

			path.Dispose();
		}

		private void stopwatch_PrepaintForeground(object sender, PrepaintEventArgs e)
		{
			e.CancelDefaultPainting = true;
		}

		private void button1_Click(object sender, EventArgs e)
		{
			if (timer1.Enabled)
			{
				UpdateTime();
				UpdatePointers();

				timer1.Stop();
				button1.Text = "Start";
			}
			else
			{
				button1.Text = "Stop";
				startTime = DateTime.Now;
				timer1.Start();
			}
		}

		private void button2_Click(object sender, EventArgs e)
		{
			elapsedTime = TimeSpan.Zero;
			UpdatePointers();
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			UpdateTime();
			UpdatePointers();
		}


		private DateTime startTime;
		private TimeSpan elapsedTime;
	}
}
